#include "eval_stack.hh"
#include <stdexcept>
#include <iostream>

using namespace std;

//======================================  Eval stack class
bool 
eval_token::getLogical(void) const {
    throw logic_error("Boolean data types not implemented");
}

double 
eval_token::getNumeric(void) const {
    throw logic_error("Numeric data types not implemented");
}

void 
eval_token::setLogical(bool yorn) {
    throw logic_error("Boolean data types not implemented");
}

void 
eval_token::setNumeric(double val) {
    throw logic_error("Numeric data types not implemented");
}

//======================================  Destroy Eval stack
eval_stack::~eval_stack(void) {
    clear();
}

//======================================  Eval stack Dump
void 
eval_stack::dump(std::ostream& out) const {
    out << "-----------------  Stack Dump, size=" << size() << endl;
    for (unsigned int inx=size(); inx; ) {
        out << --inx << "  type=" << mVect[inx]->getType() << "  "
	    << mVect[inx]->getString() << endl;
    }
}

//======================================  Perform operation on the top elements
//                                        of the stack.
void 
eval_stack::evaluate(OpsEnum op) {
    throw logic_error("Stack operations aren't defined.");
}

//======================================  Eval stack push operators
void 
eval_stack::push(const eval_token& tkn) {
    push(tkn.clone());
}

void 
eval_stack::push(eval_token* tkn) {
    mVect.push_back(tkn);
}

void 
eval_stack::push(const string& str) {
    throw logic_error("push(string) not implemented");
}

void 
eval_stack::push(double val) {
    throw logic_error("push(double) not implemented");
}

void 
eval_stack::push(bool val) {
    throw logic_error("push(bool) not implemented");
}


//======================================  Eval stack pop operators
bool 
eval_stack::pop_logical(void) {
    if (empty()) throw logic_error("Attempt to pop off empty stack");
    bool rc = top().getLogical();
    remove();
    return rc;
}

double 
eval_stack::pop_numeric(void) {
    if (empty()) throw logic_error("Attempt to pop off empty stack");
    double rc = top().getNumeric();
    remove();
    return rc;
}

std::string 
eval_stack::pop_string(void) {
    if (empty()) throw logic_error("Attempt to pop off empty stack");
    string r = top().getString();
    remove();
    return r;
}

//======================================  Eval stack peek operators
const eval_token&
eval_stack::peek(int off) const {
    int inx = mVect.size();
    if (off <= 0 || off > inx) throw logic_error("Invalid offset");
    inx -= off;
    return *mVect[inx];
}

eval_token::datype 
eval_stack::type(int off) const {
    int inx = mVect.size();
    if (off <= 0 || off > inx) throw logic_error("Invalid offset");
    inx -= off;
    return mVect[inx]->getType();
}
