#ifndef HTML_STYLE_HH
#define HTML_STYLE_HH

#include <string>
#include "html/Attrib.hh"

namespace html {
  class color;
  class font;
  class size;
  class writer;

  /**  Class style contains text style attributes.
    *  @memo Text style attribute.
    */
  class StyleClass : public attList {
  public:
    /**  Construct a style specifier.
      */
    StyleClass(void);
    StyleClass(const StyleClass& x);
    ~StyleClass(void);
    void putAttr(writer& w) const;
    StyleClass& setColor(const color& c);
    StyleClass& setFace(const font& c);
    StyleClass& setSize(const size& c);
    StyleClass& setWeight(const std::string& c);
  };

  /**  Class style contains text style attributes.
    *  @memo Text style attribute.
    */
  class style {
  public:
    /**  Construct a style specifier.
      */
    style(void);
    style(const style& x);
    ~style(void);
    void addClass(const std::string& name, const StyleClass& c);
    void write(writer& w) const;
    bool empty(void) const;

  private:
    typedef std::map<std::string, StyleClass> StyleDict;
    typedef StyleDict::iterator style_iterator;
    typedef StyleDict::const_iterator const_style_iterator;

    StyleDict mDict;
  };

}

inline bool
html::style::empty(void) const {
    return mDict.empty();
}

#endif //  HTML_STYLE_HH
