#include "lmsg/TransOutput.hh"
#include "lmsg/Grinder.hh"
#include "lmsg/Buffer.hh"
#include "Time.hh"

static lmsg::Grinder grub;

lmsg::TransOutput::TransOutput(Buffer& b) {
    setBuffer(b);
}

void
lmsg::TransOutput::setBuffer(Buffer& b) {
    mData   = b.getDataAddr();
    mIndex  = 0;
    mLength = b.getDataSize();
}

lmsg::TransOutput::TransOutput(void) 
  : mData(0), mIndex(0), mLength(0)
{
}

lmsg::size_type 
lmsg::TransOutput::write(const char* x, size_type N) throw(std::runtime_error) {
    align(sizeof(char));
    if (mLength == 0) {
        mIndex += N;
    } else if (mIndex+N > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	memcpy(mData+mIndex, x, N);
	mIndex += N;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const std::string* x, size_type N) throw(std::runtime_error) {
    for (size_type i=0 ; i< N ; i++) {
        short Ls = x->size();
        write(&Ls);
	write(x->c_str(), Ls);
	x++;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const short* x, size_type N) throw(std::runtime_error) {
    align(sizeof(short));
    if (!N) return 0;
    size_type lBytes = N*sizeof(short);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	char* out = mData+mIndex;
	grub.swap2((const char**)&x, &out, N);
	mIndex += lBytes;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const int* x, size_type N) throw(std::runtime_error) {
    align(sizeof(int));
    if (!N) return 0;
    size_type lBytes = N*sizeof(int);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	char* out = mData+mIndex;
	grub.swap4((const char**)&x, &out, N);
	mIndex += lBytes;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const long* x, size_type N) throw(std::runtime_error) {
    align(sizeof(long));
    if (!N) return 0;
    size_type lBytes = N*sizeof(long);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	char* out = mData+mIndex;
	grub.swap4((const char**)&x, &out, N);
	mIndex += lBytes;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const float* x, size_type N) throw(std::runtime_error) {
    align(sizeof(float));
    if (!N) return 0;
    size_type lBytes = N*sizeof(float);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
        char* out = mData+mIndex;
	grub.swap4((const char**)&x, &out, N);
	mIndex += lBytes;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const double* x, size_type N) throw(std::runtime_error) {
    align(sizeof(double));
    if (!N) return 0;
    size_type lBytes = N*sizeof(double);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	char* out = mData+mIndex;
	grub.swap8((const char**)&x, &out, N);
	mIndex += lBytes;
    }
    return N;
}

lmsg::size_type 
lmsg::TransOutput::write(const Time* x, size_type N) throw(std::runtime_error) {
    align(sizeof(Time::ulong_t));
    if (!N) return 0;
    size_type lBytes = 2*N*sizeof(Time::ulong_t);
    if (mLength == 0) {
        mIndex += lBytes;
    } else if (mIndex+lBytes > mLength) {
        throw std::overflow_error("Buffer overflow");
    } else {
	char* out = mData+mIndex;
	grub.swap4((const char**)&x, &out, 2*N);
	mIndex += lBytes;
    }
    return N;
}

void
lmsg::TransOutput::align(size_type N) {
    size_type nByte = (-mIndex) & (N-1);
    mIndex += nByte;
}

