/***************************************************************************
    File        : Parameter.cpp
    Description : Implements the class Parameter
 ---------------------------------------------------------------------------
    Begin       : Tue Mar 22 2005
    Author(s)   : Roberto Grosso
 ***************************************************************************/
 
#include "Parameter.h"


// ReadParameters
//  Read the Noise Floor parameters from file
void
gwd::Parameter::ReadParameters(const std::string& filename)
{
  int count = 0;
  std::ifstream theFile;

  theFile.open(filename.c_str(),std::ios::in);
  if (!(theFile.is_open()) || theFile.fail())
  {
    char buf[1024];
    sprintf(buf,"can't open file %s",filename.c_str());
    std::cout << "ERROR: " << buf << std::endl;
  }
  else
  {
    // read Sensor mode
    while(!theFile.eof())
    {
      // Read one complete line at a time
      char buffer[1024];
      theFile.getline(buffer,1024);

      // Check if it is an empty line
      if (buffer[0] == '\n')
        continue;

      // remove leading empty spaces
      char* tmpch = &buffer[0];
      while (*tmpch == ' ')
        tmpch++;
      // Check if it is an empty line, a zero or a comment line
      if (*tmpch == '\n' || *tmpch == '\0' || *tmpch == '#')
        continue;

      // Read the parameters
      //string s = "34 22";
      std::string keyword;
      std::string eqs;
      bool flag = false;
      std::string s = tmpch;
      std::istringstream ins; // Declare an input string stream.  
      ins.str(s);        // Specify string to read.
      ins >> keyword;     // Reads the integers from the string.
      if (keyword == "DETECTOR_CHANNEL")
      { if (!(ins >> eqs) || !(ins >> mChannel)) flag = true; }
      else if (keyword == "NOTCH_FILTER_ORDER")
      { if (!(ins >> eqs) || !(ins >> mOrderNotchFilter)) flag = true; }
      else if (keyword == "UPDATE_WHITENING_FILTER")
      { if (!(ins >> eqs) || !(ins >> mUpdateWhiteFlt)) flag = true; }  
      else if (keyword == "ORDER_WHITENING_FILTER")
      { if (!(ins >> eqs) || !(ins >> mOrderWhiteFilter)) flag = true; }
      else if (keyword == "FREQUENCIES_PSD")
      { if (!(ins >> eqs) || !(ins >> mNoOfFreqPsd)) flag = true; }
      else if (keyword == "SIZE_PSD")
      { if (!(ins >> eqs) || !(ins >> mSizePsd)) flag = true; }
      else if (keyword == "RM_FILTER_FOR_WHITENING_FILTER")
      { if (!(ins >> eqs) || !(ins >> mRMSizeForWhiteningFlt)) flag = true; }
      else if (keyword == "SIZE_RM_FILTER")
      { if (!(ins >> eqs) || !(ins >> mWindowSizeRunningMedian)) flag = true; }
      else if (keyword == "FLT_ORDER")
      { if (!(ins >> eqs) || !(ins >> mFltOrder)) flag = true; }
      else if (keyword == "CUTOFF_FREQ")
      { if (!(ins >> eqs) || !(ins >> mUpperCutoffFreq)) flag = true; }
      else if (keyword == "BAND_FLT_ORDER")
      { if (!(ins >> eqs) || !(ins >> mBandFltOrder)) flag = true; }
      else if (keyword == "BAND_FREQ_A")
      { if (!(ins >> eqs) || !(ins >> mBandFreqA)) flag = true; }
      else if (keyword == "BAND_FREQ_B")
      { if (!(ins >> eqs) || !(ins >> mBandFreqB)) flag = true; }
      else if (keyword == "BAND_FREQ_C")
      { if (!(ins >> eqs) || !(ins >> mBandFreqC)) flag = true; }
      else if (keyword == "TIME_STRIDE")
      { if (!(ins >> eqs) || !(ins >> mTimeStride)) flag = true; }
      else
      {
        std::cout << "ERROR: unknown keyword " << keyword << " in parameter file: " << filename << std::endl;
        exit(1);
      }

      if (flag)
      {
        std::string message = "Check sintax for paramter " + keyword + "  in parameter file " + filename;
        std::cout << message << std::endl;
        exit(1);
      }
      else
        count++;
    }
  }
  theFile.close();

  // Check if all parameters were read
  if (count != 15)
  {
    std::string message = "ERROR: too many or too few parameters are specified in parameter file " + filename + "\n 14 parameters are required, read Monitor description for more information";
    std::cout << message << std::endl;
    exit(1);
  }

}

