// -*- C++

//@{

/** @name ServoMon User's Guide

\begin{center}
{\bf{\Large Defining Triggers for the Servo Monitor in the Data Monitor Tool}}
\end{center}

\begin{center}
{\bf{\large Abstract}}
\end{center}

This document is a guide to defining triggers in the Data Monitor Tool.
ServoMon reads definitions of triggers from a text configuration file,
and in the ProcessData() loop, decides if a trigger should be sent to the
Trigger Manager.  The configuration file will specify conditions under
which a trigger event may be sent to the Trigger Manager.

\begin{center}
{\bf{\large Introduction}}
\end{center}

The ServoMon monitor monitors channels specified by the user, and generates 
    triggers when the data in the channel satisfies certain conditions
    (Operating State Conditions, or OSCs).

    ServoMon consists of some number of trigger objects. Each trigger object
    has certain parameters and settings associated to it, the most important
    being the OSC which defines when that trigger will be generated.

    The OSC associated with the trigger must be defined in a separate
    configuration file in OperStateCond format.

    \begin{center}
{\bf{\large Configuration File Format}}
\end{center}

To understand the ServoMon configuration file format, we shall describe in
    some detail the trigger objects.

    Triggers can be of two types: 'instantaneous', or 'zero-order hold (ZOH)'.
    The 'instantaneous' triggers, in general, generate and send a trigger
    the instant its OSC is satisfied.  The 'ZOH' triggers, on the other hand,
    generate and send a trigger the instant its OSC is satisfied, and keeps
    generating and sending triggers for a user-specified amount of time
    called the 'hold duration', hence the name ZOH.  In addition to the
    'hold duration', a ZOH trigger has a 'deadtime' which is specified by
    the user.  This specifies the minimum amount of time that must lapse
    after the end of the hold duration before the trigger can be
        generated and sent.

    For 'instantaneous' triggers, one can control the frequency at
    which any Epics alarms and/or MetaDB entries are generated.  These are
        defined by integers: generation of Epics and MetaDB triggers occur
        at most once every N * Stride, where N is the integer referred to
        above.

        An example configuration file:
\begin{verbatim}
#
# An example configuration file for ServoMon
#
test
interval 2.0
logfile ServoMon.log
trigger                # this is an error, and will be ignored
trigger foobar         # this is also an error and will be ignored
trigger barfoo LVEA_quiet warn epics 10 meta 15
# trigger shhhh LVEA_quiet warn meta 200
trigger click Arm_locked warn epics 18
trigger eureka cubaan warn epics 18 meta 69
#
# The next 2 are legal, but the Epics and MetaDB intervals
# are not used: they just decide if Epics alarms or MetaDB entries
# are made
#
# trigger <trigger name> <OSC name> <warning level> [epics <epics interval>]\\ 
#        [meta <metaDB interval>] [hold <hold duration>] \\
#        [deadtime <deadtime interval>]
#
trigger inout_good  snafu  warn epics 10 meta 15 hold 17 deadtime 49
trigger inout_thethird  snafu  warn hold 72       # this is a comment

\end{verbatim}
        

*/

//@}

