#include "NameMsg.hh"
#include "lmsg/TransInput.hh"
#include "lmsg/TransOutput.hh"
#include <netinet/in.h>

using namespace std;
using namespace lmsg;

//======================================  Default constructor
NameData::NameData(void) {
}

//======================================  Data constructor
NameData::NameData(const string& Name, const MsgAddr& Addr, NameProcs PType)
  : mName(Name), mAddr(Addr), mPType(PType)
{
}

//======================================  Copy constructor
NameData::NameData(const NameData& msg) 
{
    mName  = msg.mName;
    mAddr  = msg.mAddr;
    mPType = msg.mPType;
}

//======================================  Destructor
NameData::~NameData(void) {}

//======================================  Copy constructor
NameData&
NameData::operator=(const NameData& msg) 
{
    mName  = msg.mName;
    mAddr  = msg.mAddr;
    mPType = msg.mPType;
    return *this;
}

//======================================  Name data size
size_type 
NameData::size(void) const {
    return sizeof(short) + mName.size() + sizeof(mAddr) + sizeof(mPType);
}

//======================================  Get the name field
const char* 
NameData::getName(void) const {
    return mName.c_str();
}

//======================================  Get the address field
const MsgAddr& 
NameData::getAddr(void) const {
    return mAddr;
}
 
//======================================  Set the address field
void
NameData::setAddr(const MsgAddr& addr) {
    mAddr = addr;
}

//======================================  Get process type field
NameProcs 
NameData::getPType(void) const {
    return mPType;
}

//======================================  Fix incomplete Addresses
void 
NameData::fixAddr(const MsgAddr& from) {
    if (!mAddr.getIPAddr() || mAddr.getIPAddr() == INADDR_LOOPBACK) {
        mAddr.setIPAddr(from.getIPAddr());
    }
}

//======================================  MsgAddr output utility function.
TransOutput& 
operator<<(TransOutput& out, const MsgAddr& addr) {
    int   ipaddr = addr.getIPAddr();
    short ipport = addr.getIPPort();
    short sbproc = addr.getSubProcess();
    out << ipaddr << ipport << sbproc;
    return out;
}

//======================================  MsgAddr input utility function
TransInput& 
operator>>(TransInput& in, MsgAddr& addr) {
    int   ipaddr;
    short ipport, sbproc;
    in >> ipaddr >> ipport >> sbproc;
    addr = MsgAddr(ipaddr, ipport, sbproc);
    return in;
}

//======================================  Data input function
TransInput& 
operator>>(TransInput& in, NameData& nd) {
    string name;
    MsgAddr addr;
    long type;
    in >> name >> addr >> type;
    nd = NameData(name, addr, NameProcs(type));
    return in;
}

//======================================  Data output function
TransOutput& 
operator<<(TransOutput& out, const NameData& nd) {
    out << nd.getName() << nd.getAddr() << long(nd.getPType());
    return out;
}


//======================================  DataMsg templates
template<class T, unsigned int MsgID>
void 
DataMsg<T,MsgID>::getData(TransOutput& out) const {
    out << mData;
}

template<class T, unsigned int MsgID>
void 
DataMsg<T,MsgID>::setData(TransInput& in) throw(std::runtime_error) {
    in >> mData;
}

template class DataMsg<NameData, m_addName>;
template class DataMsg<NameData, m_lookup>;
template class DataMsg<NameData, m_remName>;
template class DataMsg<NameData, m_status>;
template class DataMsg<NameData, m_index>;
template class DataMsg<int,      m_IDget>;
