#ifndef FRVECTREF_HH
#define FRVECTREF_HH

#ifndef __CINT__
#include "framecpp/Types.hh"
#else
namespace FrameCPP {
  class FrVect;
}
#endif

class DVector;


/**  Class FrVectRef contains a wrapped reference to an FrVect and Supplies 
  *  various methods to access the data.
  */
class FrVectRef {

//--------------------------------------  Lib-independent types
public:
    enum compression_mode {
      kUncompressed=0,
      kGZip=1,
      kGZipDifferential=3,
      kZeroSuppress2=5,
      kZeroSuppress4=8
    };

//-------------------------------------  Lib-dependent types
public:
  typedef unsigned long    size_type;
  typedef FrameCPP::FrVect vect_type;

//-------------------------------------  Class methods
public:
  FrVectRef(void);
  FrVectRef(const vect_type* x);
  FrVectRef(const DVector& v, double x0=0.0, double dx=1.0, 
	    const std::string& unit="ct");
  ~FrVectRef(void);
  DVector*  getDVector(void) const;
  DVector*  getDVector(size_type dim, double x0, double dx) const;
  DVector*  getDVector(size_type inx0, size_type dinx) const;
  double    getDimX0(size_type i) const;
  double    getDimXmax(size_type i) const;
  double    getDimDx(size_type i) const;
  size_type getDimNx(size_type i) const;
  size_type getIndex(size_type dim, double x0) const;
  size_type getNBits(void) const;
  void      compress(compression_mode cmode);
  vect_type* release(void);
  vect_type* replicate(void) const;
  void      setVect(const DVector& v, double x0=0.0, double dx=1.0, 
		    const std::string& unit="ct");
  void      setVect(vect_type* v, bool owner=false);
  size_type size(void) const;
  bool      empty(void) const;

private:
  bool       mOwner;
  vect_type* mVect;
};

inline FrVectRef::vect_type*
FrVectRef::release(void) {
    if (!mOwner) return replicate();
    mOwner = false;
    return mVect;
}

inline bool
FrVectRef::empty(void) const {
    return !size();
}

#endif // FRVECTREF_HH
