#include "MultiPipe.hh"

using namespace std;

//======================================  Default constructor
MultiPipe::MultiPipe(void) 
  : mGain (1) {
}

//======================================  Copy constructor
MultiPipe::MultiPipe(const MultiPipe& p) 
  : mPipe(p.mPipe.size()), mGain (p.mGain)
{
    for (unsigned int i=0 ; i<p.mPipe.size() ; i++) {
        mPipe[i] = p.mPipe[i]->clone();
    }
}

//======================================  Destructor
MultiPipe::~MultiPipe(void) {
   for (unsigned int i=0 ; i<mPipe.size() ; i++) {
      delete mPipe[i];
   }
}

//======================================  Clone
MultiPipe* 
MultiPipe::clone(void) const {
    return new MultiPipe(*this);
}

//======================================  Assignment
MultiPipe& MultiPipe::operator= (const MultiPipe& p)
{
   if (this != &p) {
      mGain = p.mGain;
      for (unsigned int i=0 ; i<p.mPipe.size() ; i++) {
         delete mPipe[i];
      }
      mPipe.clear();
      mPipe.resize(p.mPipe.size());
      for (unsigned int i=0 ; i<p.mPipe.size() ; i++) {
          mPipe[i] = p.mPipe[i]->clone();
      }
   }
   return *this;
}

//======================================  Add pipe
Pipe* 
MultiPipe::addPipe(const Pipe& p) {
    mPipe.push_back(p.clone());
    return mPipe.back();
}

//======================================  Apply
TSeries 
MultiPipe::apply(const TSeries& ts) {
    if (mPipe.empty()) return ts;
    TSeries x = mPipe[0]->apply(ts);
    for (unsigned int i=1 ; i<mPipe.size() ; i++) {
        x = mPipe[i]->apply(x);
    }
    if (mGain != 1) x *= mGain;
    return x;
}

//======================================  Data check
void 
MultiPipe::dataCheck(const TSeries& ts) const {
    if (!mPipe.empty()) mPipe[0]->dataCheck(ts);
    // for (unsigned int i=1 ; i<mPipe.size() ; i++) {
    //     mPipe[i]->dataCheck(ts);
    // }
}

//======================================  in use
bool 
MultiPipe::inUse(void) const {
    unsigned int N = mPipe.size();
    for (unsigned int i=0 ; i<N ; i++) {
        if (mPipe[i]->inUse()) return true;
    }
    return false;
}

//======================================  Start time
Time 
MultiPipe::getStartTime(void) const {
    if (mPipe.empty()) return Time(0);
    return mPipe[0]->getStartTime();
}

//======================================  current time
Time 
MultiPipe::getCurrentTime(void) const {
    if (mPipe.empty()) return Time(0);
    return mPipe[0]->getCurrentTime();
}

//======================================  get gain
double 
MultiPipe::getGain(void) const {
   return mGain;
}

//======================================  get Time delay
Interval 
MultiPipe::getTimeDelay(void) const {
    Interval delay(0.0);
    unsigned int N=mPipe.size();
    for (unsigned int i=0 ; i<N ; i++) {
        delay += mPipe[i]->getTimeDelay();
    }
    return delay;
}

//======================================  set gain
void 
MultiPipe::setGain(double gain) {
   mGain = gain;
}

//======================================  Reset
void 
MultiPipe::reset() {
    unsigned int N=mPipe.size();
    for (unsigned int i=0 ; i<N ; i++) {
        mPipe[i]->reset();
    }
    mGain = 1;
}

//======================================  Xfer
bool 
MultiPipe::xfer (fComplex& coeff, double f) const throw() {
   coeff = fComplex (mGain, 0);
   fComplex temp;
   unsigned int N=mPipe.size();
   for (unsigned int i=0 ; i<N ; i++) {
      if (!mPipe[i]->Xfer (temp, f)) {
         return false;
      }
      coeff *= temp;
   }
   return true;
}


